//
//  TableViewController.swift
//  TestTableViewSectionGlitch
//
//  Created by James Bamford on 29/01/2018.
//  Copyright © 2018 JB. All rights reserved.
//

import UIKit
import CoreData

class TableViewController: CoreDataTableViewController<RowEntity>
{
  // Change this constant to true to cause the stale section header issue to occur after you go to the
  // SimpleViewController by selecting any row in the tableView and pressing the SimpleViewController's
  // "Toggle TableView Sections" button and returning back to the main tableview. Repeat this if required
  // such that the extra section is added and then removed and notice that the extra "section 2" header
  // is left floating on the tableview over the top of some of the rows.
  // This constant only toggles the sectionHeaderHeight being set to automatic
  // Nb. the button "Toggle TableView Sections" on this tableview performs the same toggling of sections
  // but the table correct animates and doesn't leave an extra "section 2" header behind, although if you
  // do the above repro with the SimpleViewController to get the extra header then the onscreen toggling of
  // sections with this tableview's button shows the table animating with the extra "section 2" header left
  // floating on the tableview
  // Also note that as well as causing this floating section header problem, setting this constant to true
  // causes the animation when onscreen toggling of the sections to not work correctly, the section 2 header doesn't
  // animate down with the rows rather it appears with the rows moving through the section header
  private let setAutomaticDimensionSectionHeaderHeight=true;
  
  @IBAction func toggleSectionsButtonPressed(_ sender: Any)
  {
    dbgLog("")
    TableViewController.toggleSections(context: AppDelegate.viewContext)
  }
  
  func createSectionsIfNeeded(context:NSManagedObjectContext)
  {
    let sectionsArray = TableViewController.getSections(context: context)
    
    if (sectionsArray.count == 0)
    {
      TableViewController.makeSection("section 0", true, 1, AppDelegate.viewContext)
      
      // start with just 4 rows in section B
      TableViewController.makeSection("section 2", false, 4, AppDelegate.viewContext)
      
      do
      {
        try context.save()
      }
      catch let error
      {
        dbgLog("failed to save context; error=\(error)")
      }
    }
  }
  
  private func createFetchRequest()
  {
    let context = AppDelegate.viewContext
    let request: NSFetchRequest<RowEntity> = RowEntity.fetchRequest()
    request.sortDescriptors = [
      NSSortDescriptor(key: "section.name", ascending: true, selector: #selector(NSString.localizedCaseInsensitiveCompare(_:))),
      NSSortDescriptor(key: "name", ascending: true, selector: #selector(NSString.localizedCaseInsensitiveCompare(_:))),
    ]
    
    fetchedResultsController = NSFetchedResultsController<RowEntity>(
      fetchRequest: request,
      managedObjectContext: context,
      sectionNameKeyPath: "section.name",
      cacheName: nil)
  }
  
  override func viewDidLoad()
  {
    super.viewDidLoad()
    
    // Get dynamic section heights working
    
    // its this that seems to be causing the rogue header bug.. setting our section height to automatic dimension!
    if (setAutomaticDimensionSectionHeaderHeight)
    {
      tableView.sectionHeaderHeight = UITableViewAutomaticDimension // this must be the default as dynamic section headers work without this so long as there's an estimated section header height specified
    }
    tableView.estimatedSectionHeaderHeight = 50.0 // seems like this property is absolutely crucial to getting auto layout sized section headers working.. probably need something equivalent set to get cells to do the same?
    
    // Do any additional setup after loading the view, typically from a nib.
    createSectionsIfNeeded(context: AppDelegate.viewContext)

    createFetchRequest()
  }
  
  
  override func viewWillAppear(_ animated: Bool)
  {
    dbgLog("");
    /*
    // functionality I've added to my main app in attempting to fix this floating section header problem. Not needed to repro the issue so removed
    do
    {
      try fetchedResultsController!.performFetch()
    }
    catch let error
    {
      dbgLog("failed to performFetch; error=\(error)")
    }

    tableView.reloadData()
    */
  }

  override func didReceiveMemoryWarning()
  {
    super.didReceiveMemoryWarning()
    // Dispose of any resources that can be recreated.
  }

  // MARK: Tableview Data Source
  
  // This function setting automatic dimension can reproduce the issue the same as initialising the sectionHeaderHeight to automatic
  /*
  override func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat
  {
    if (section==0)
    {
      return 0.0  // hide the section header for the extra section
    }

    return UITableViewAutomaticDimension;
  }
  */
  
  
  override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String?
  {
    /*
    // functionality from my main app behaviour. Not needed for this repro so removed
    if (section == 0)
    {
      return nil
    }
    */
    
    let rowEntity = fetchedResultsController!.object(at: IndexPath(row: 0, section: section))
    if let sectionEntity = rowEntity.section
    {
      return sectionEntity.name
    }
    return "<Unknown>"
  }

/*
  override func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView?
  {
    return nil
  }
 */

  override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
  {
    let rowEntity = fetchedResultsController?.object(at: indexPath)
    let cell = tableView.dequeueReusableCell(withIdentifier: "cell")
    cell!.textLabel?.text = rowEntity?.name
    
    return cell!
  }
  
  override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath)
  {
    tableView.deselectRow(at: indexPath, animated: true)
  }
  
  /*
  // functionality from my main app behaviour. Not needed for this repro so removed
  override func sectionIndexTitles(for tableView: UITableView) -> [String]?
  {
    // disable index being shown for now by having no index titles
    return nil
  }
 */
  
  // MARK: Static helper funcs to build tableview sections
  static func makeSection(_ sectionName:String, _ extraSection:Bool, _ numberOfRows:Int, _ context:NSManagedObjectContext)
  {
    let sectionEntity = SectionEntity(context: context)
    sectionEntity.name = sectionName
    
    for i in 0..<numberOfRows
    {
      let rowEntity = RowEntity(context: context)
      if (extraSection)
      {
        rowEntity.name = "Extra Section \(sectionName): Row \(i)"
      }
      else
      {
        rowEntity.name = "Section \(sectionName): Row \(i)"
      }
      rowEntity.section = sectionEntity
    }
  }
  
  static func removeSection(context:NSManagedObjectContext)
  {
    let sectionsArray = getSections(context: context)
    context.delete(sectionsArray[1])
  }
  
  static func getSections(context:NSManagedObjectContext) -> [SectionEntity]
  {
    let request: NSFetchRequest<SectionEntity> = SectionEntity.fetchRequest()
    request.sortDescriptors = [ NSSortDescriptor(key: "name", ascending: true, selector: #selector(NSString.localizedCaseInsensitiveCompare(_:))) ]
    var sectionsArray = [SectionEntity]()
    
    do
    {
      sectionsArray = try context.fetch(request)
    }
    catch let error
    {
      dbgLog("failed to execute fetch request for sections for error=\(error)")
    }
    
    return sectionsArray
  }
  
  static func toggleSections(context:NSManagedObjectContext)
  {
    let sectionsArray = getSections(context: context)
    if (sectionsArray.count == 2)
    {
      // create section 1 with 3 rows
      makeSection("section 1", false, 3, AppDelegate.viewContext)
      dbgLog("creating section 1")
    }
    else
    {
      removeSection(context: AppDelegate.viewContext)
      dbgLog("deleting section 1")
    }
    
    do
    {
      try context.save()
    }
    catch let error
    {
      dbgLog("failed to save context; error=\(error)")
    }
  }
}

